@@ -5,9 +5,10 @@ import sys
 import types
 import inspect
 import builtins
+import warnings
 from collections import Mapping
 
-from xonsh import ast
+from xonsh.ast import CtxAwareTransformer
 from xonsh.parser import Parser
 from xonsh.tools import subproc_toks, find_next_break
 from xonsh.built_ins import load_builtins, unload_builtins
@@ -38,21 +39,21 @@ class Execer(object):
         self.filename = filename
         self.debug_level = debug_level
         self.unload = unload
-        self.ctxtransformer = ast.CtxAwareTransformer(self.parser)
+        self.ctxtransformer = CtxAwareTransformer(self.parser)
         load_builtins(execer=self, config=config, login=login, ctx=xonsh_ctx)
 
     def __del__(self):
         if self.unload:
             unload_builtins()
 
-    def parse(self, input, ctx, mode='exec'):
+    def parse(self, input, ctx, mode='exec', transform=True):
         """Parses xonsh code in a context-aware fashion. For context-free
-        parsing, please use the Parser class directly.
+        parsing, please use the Parser class directly or pass in
+        transform=False.
         """
-        if ctx is None:
-            ctx = set()
-        elif isinstance(ctx, Mapping):
-            ctx = set(ctx.keys())
+        if not transform:
+            return self.parser.parse(input, filename=self.filename, mode=mode,
+                                     debug_level=(self.debug_level > 1))
 
         # Parsing actually happens in a couple of phases. The first is a
         # shortcut for a context-free parser. Normally, all subprocess
@@ -78,11 +79,15 @@ class Execer(object):
         # (ls) is part of the execution context. If it isn't, then we will
         # assume that this line is supposed to be a subprocess line, assuming
         # it also is valid as a subprocess line.
+        if ctx is None:
+            ctx = set()
+        elif isinstance(ctx, Mapping):
+            ctx = set(ctx.keys())
         tree = self.ctxtransformer.ctxvisit(tree, input, ctx, mode=mode)
         return tree
 
     def compile(self, input, mode='exec', glbs=None, locs=None, stacklevel=2,
-                filename=None):
+                filename=None, transform=True):
         """Compiles xonsh code into a Python code object, which may then
         be execed or evaled.
         """
@@ -93,13 +98,20 @@ class Execer(object):
             glbs = frame.f_globals if glbs is None else glbs
             locs = frame.f_locals if locs is None else locs
         ctx = set(dir(builtins)) | set(glbs.keys()) | set(locs.keys())
-        tree = self.parse(input, ctx, mode=mode)
+        tree = self.parse(input, ctx, mode=mode, transform=transform)
         if tree is None:
             return None  # handles comment only input
-        code = compile(tree, filename, mode)
+        if transform:
+            with warnings.catch_warnings():
+                # we do some funky things with blocks that cause warnings
+                warnings.simplefilter('ignore', SyntaxWarning)
+                code = compile(tree, filename, mode)
+        else:
+            code = compile(tree, filename, mode)
         return code
 
-    def eval(self, input, glbs=None, locs=None, stacklevel=2):
+    def eval(self, input, glbs=None, locs=None, stacklevel=2,
+                transform=True):
         """Evaluates (and returns) xonsh code."""
         if isinstance(input, types.CodeType):
             code = input
@@ -108,12 +120,14 @@ class Execer(object):
                                 glbs=glbs,
                                 locs=locs,
                                 mode='eval',
-                                stacklevel=stacklevel)
+                                stacklevel=stacklevel,
+                                transform=transform)
         if code is None:
             return None  # handles comment only input
         return eval(code, glbs, locs)
 
-    def exec(self, input, mode='exec', glbs=None, locs=None, stacklevel=2):
+    def exec(self, input, mode='exec', glbs=None, locs=None, stacklevel=2,
+                transform=True):
         """Execute xonsh code."""
         if isinstance(input, types.CodeType):
             code = input
@@ -122,7 +136,8 @@ class Execer(object):
                                 glbs=glbs,
                                 locs=locs,
                                 mode=mode,
-                                stacklevel=stacklevel)
+                                stacklevel=stacklevel,
+                                transform=transform)
         if code is None:
             return None  # handles comment only input
         return exec(code, glbs, locs)
