@@ -3,14 +3,20 @@
 import os
 import sys
 import time
+import ctypes
 import signal
 import builtins
-from subprocess import TimeoutExpired, check_output
-from collections import deque
+import functools
+import subprocess
+import collections
 
+from xonsh.lazyasd import LazyObject
 from xonsh.platform import ON_DARWIN, ON_WINDOWS, ON_CYGWIN
 
-tasks = deque()
+tasks = LazyObject(collections.deque, globals(), 'tasks')
+# Track time stamp of last exit command, so that two consecutive attempts to
+# exit can kill all jobs and exit.
+_last_exit_time = None
 
 
 if ON_DARWIN:
@@ -19,14 +25,34 @@ if ON_DARWIN:
         # any zombie processes in the process group.
         # See github issue #1012 for details
         for pid in job['pids']:
+            if pid is None:  # the pid of an aliased proc is None
+                continue
             os.kill(pid, signal)
-
 elif ON_WINDOWS:
     pass
-
+elif ON_CYGWIN:
+    # Similar to what happened on OSX, more issues on Cygwin
+    # (see Github issue #514).
+    def _send_signal(job, signal):
+        try:
+            os.killpg(job['pgrp'], signal)
+        except Exception:
+            for pid in job['pids']:
+                try:
+                    os.kill(pid, signal)
+                except:
+                    pass
 else:
     def _send_signal(job, signal):
-        os.killpg(job['pgrp'], signal)
+        pgrp = job['pgrp']
+        if pgrp is None:
+            for pid in job['pids']:
+                try:
+                    os.kill(pid, signal)
+                except:
+                    pass
+        else:
+            os.killpg(job['pgrp'], signal)
 
 
 if ON_WINDOWS:
@@ -34,7 +60,8 @@ if ON_WINDOWS:
         job['status'] = "running"
 
     def _kill(job):
-        check_output(['taskkill', '/F', '/T', '/PID', str(job['obj'].pid)])
+        subprocess.check_output(['taskkill', '/F', '/T', '/PID',
+                                 str(job['obj'].pid)])
 
     def ignore_sigtstp():
         pass
@@ -48,25 +75,19 @@ if ON_WINDOWS:
         suspended by ctrl-z.
         """
         _clear_dead_jobs()
-
         active_task = get_next_task()
-
         # Return when there are no foreground active task
         if active_task is None:
             return
-
         obj = active_task['obj']
-
         _continue(active_task)
-
         while obj.returncode is None:
             try:
                 obj.wait(0.01)
-            except TimeoutExpired:
+            except subprocess.TimeoutExpired:
                 pass
             except KeyboardInterrupt:
                 _kill(active_task)
-
         return wait_for_active_job()
 
 else:
@@ -80,35 +101,69 @@ else:
         signal.signal(signal.SIGTSTP, signal.SIG_IGN)
 
     def _set_pgrp(info):
+        pid = info['pids'][0]
+        if pid is None:
+            # occurs if first process is an alias
+            info['pgrp'] = None
+            return
         try:
-            info['pgrp'] = os.getpgid(info['obj'].pid)
+            info['pgrp'] = os.getpgid(pid)
         except ProcessLookupError:
-            pass
+            info['pgrp'] = None
 
     _shell_pgrp = os.getpgrp()
 
-    _block_when_giving = (signal.SIGTTOU, signal.SIGTTIN, signal.SIGTSTP)
-
-    def _give_terminal_to(pgid):
-        # over-simplified version of:
-        #    give_terminal_to from bash 4.3 source, jobs.c, line 4030
-        # this will give the terminal to the process group pgid
-        if _shell_tty is not None and os.isatty(_shell_tty):
-            if not ON_CYGWIN:
-                oldmask = signal.pthread_sigmask(signal.SIG_BLOCK,
-                                                 _block_when_giving)
-            os.tcsetpgrp(_shell_tty, pgid)
-            if not ON_CYGWIN:
-                signal.pthread_sigmask(signal.SIG_SETMASK, oldmask)
+    _block_when_giving = LazyObject(lambda: (signal.SIGTTOU, signal.SIGTTIN,
+                                             signal.SIGTSTP, signal.SIGCHLD),
+                                    globals(), '_block_when_giving')
 
     # check for shell tty
-    try:
-        _shell_tty = sys.stderr.fileno()
-        if os.tcgetpgrp(_shell_tty) != os.getpgid(os.getpid()):
-            # we don't own it
-            _shell_tty = None
-    except OSError:
-        _shell_tty = None
+    @functools.lru_cache(1)
+    def _shell_tty():
+        try:
+            _st = sys.stderr.fileno()
+            if os.tcgetpgrp(_st) != os.getpgid(os.getpid()):
+                # we don't own it
+                _st = None
+        except OSError:
+            _st = None
+        return _st
+
+
+    # _give_terminal_to is a simplified version of:
+    #    give_terminal_to from bash 4.3 source, jobs.c, line 4030
+    # this will give the terminal to the process group pgid
+    if ON_CYGWIN:
+        _libc = LazyObject(lambda: ctypes.CDLL('cygwin1.dll'),
+                           globals(), '_libc')
+
+        # on cygwin, signal.pthread_sigmask does not exist in Python, even
+        # though pthread_sigmask is defined in the kernel.  thus, we use
+        # ctypes to mimic the calls in the "normal" version below.
+        def _give_terminal_to(pgid):
+            st = _shell_tty()
+            if st is not None and os.isatty(st):
+                omask = ctypes.c_ulong()
+                mask = ctypes.c_ulong()
+                _libc.sigemptyset(ctypes.byref(mask))
+                for i in _block_when_giving:
+                    _libc.sigaddset(ctypes.byref(mask), ctypes.c_int(i))
+                _libc.sigemptyset(ctypes.byref(omask))
+                _libc.sigprocmask(ctypes.c_int(signal.SIG_BLOCK),
+                                  ctypes.byref(mask),
+                                  ctypes.byref(omask))
+                _libc.tcsetpgrp(ctypes.c_int(st), ctypes.c_int(pgid))
+                _libc.sigprocmask(ctypes.c_int(signal.SIG_SETMASK),
+                                  ctypes.byref(omask), None)
+    else:
+        def _give_terminal_to(pgid):
+            st = _shell_tty()
+            if st is not None and os.isatty(st):
+                oldmask = signal.pthread_sigmask(signal.SIG_BLOCK,
+                                                 _block_when_giving)
+                os.tcsetpgrp(st, pgid)
+                signal.pthread_sigmask(signal.SIG_SETMASK, oldmask)
+
 
     def wait_for_active_job():
         """
@@ -116,21 +171,17 @@ else:
         suspended by ctrl-z.
         """
         _clear_dead_jobs()
-
         active_task = get_next_task()
-
         # Return when there are no foreground active task
         if active_task is None:
             _give_terminal_to(_shell_pgrp)  # give terminal back to the shell
             return
-
-        pgrp = active_task['pgrp']
+        pgrp = active_task.get('pgrp', None)
         obj = active_task['obj']
         # give the terminal over to the fg process
-        _give_terminal_to(pgrp)
-
+        if pgrp is not None:
+            _give_terminal_to(pgrp)
         _continue(active_task)
-
         _, wcode = os.waitpid(obj.pid, os.WUNTRACED)
         if os.WIFSTOPPED(wcode):
             print()  # get a newline because ^Z will have been printed
@@ -142,7 +193,6 @@ else:
         else:
             obj.returncode = os.WEXITSTATUS(wcode)
             obj.signal = None
-
         return wait_for_active_job()
 
 
@@ -176,18 +226,20 @@ def _clear_dead_jobs():
         del builtins.__xonsh_all_jobs__[job]
 
 
-def print_one_job(num):
+def print_one_job(num, outfile=sys.stdout):
     """Print a line describing job number ``num``."""
     try:
         job = builtins.__xonsh_all_jobs__[num]
     except KeyError:
         return
+    pos = '+' if tasks[0] == num else '-' if tasks[1] == num else ' '
     status = job['status']
     cmd = [' '.join(i) if isinstance(i, list) else i for i in job['cmds']]
     cmd = ' '.join(cmd)
     pid = job['pids'][-1]
     bg = ' &' if job['bg'] else ''
-    print('[{}] {}: {}{} ({})'.format(num, status, cmd, bg, pid))
+    print('[{}]{} {}: {}{} ({})'.format(num, pos, status, cmd, bg, pid),
+          file=outfile)
 
 
 def get_next_job_number():
@@ -214,6 +266,56 @@ def add_job(info):
         print_one_job(num)
 
 
+def clean_jobs():
+    """Clean up jobs for exiting shell
+
+    In non-interactive mode, kill all jobs.
+
+    In interactive mode, check for suspended or background jobs, print a
+    warning if any exist, and return False. Otherwise, return True.
+    """
+    jobs_clean = True
+    if builtins.__xonsh_env__['XONSH_INTERACTIVE']:
+        _clear_dead_jobs()
+
+        if builtins.__xonsh_all_jobs__:
+            global _last_exit_time
+            if builtins.__xonsh_history__.buffer:
+                last_cmd_start = builtins.__xonsh_history__.buffer[-1]['ts'][0]
+            else:
+                last_cmd_start = None
+
+            if (_last_exit_time and last_cmd_start and
+                    _last_exit_time > last_cmd_start):
+                # Exit occurred after last command started, so it was called as
+                # part of the last command and is now being called again
+                # immediately. Kill jobs and exit without reminder about
+                # unfinished jobs in this case.
+                kill_all_jobs()
+            else:
+                if len(builtins.__xonsh_all_jobs__) > 1:
+                    msg = 'there are unfinished jobs'
+                else:
+                    msg = 'there is an unfinished job'
+
+                if builtins.__xonsh_env__['SHELL_TYPE'] != 'prompt_toolkit':
+                    # The Ctrl+D binding for prompt_toolkit already inserts a
+                    # newline
+                    print()
+                print('xonsh: {}'.format(msg), file=sys.stderr)
+                print('-'*5, file=sys.stderr)
+                jobs([], stdout=sys.stderr)
+                print('-'*5, file=sys.stderr)
+                print('Type "exit" or press "ctrl-d" again to force quit.',
+                      file=sys.stderr)
+                jobs_clean = False
+                _last_exit_time = time.time()
+    else:
+        kill_all_jobs()
+
+    return jobs_clean
+
+
 def kill_all_jobs():
     """
     Send SIGKILL to all child processes (called when exiting xonsh).
@@ -223,7 +325,7 @@ def kill_all_jobs():
         _kill(job)
 
 
-def jobs(args, stdin=None):
+def jobs(args, stdin=None, stdout=sys.stdout, stderr=None):
     """
     xonsh command: jobs
 
@@ -231,7 +333,7 @@ def jobs(args, stdin=None):
     """
     _clear_dead_jobs()
     for j in tasks:
-        print_one_job(j)
+        print_one_job(j, outfile=stdout)
     return None, None
 
 
@@ -240,7 +342,8 @@ def fg(args, stdin=None):
     xonsh command: fg
 
     Bring the currently active job to the foreground, or, if a single number is
-    given as an argument, bring that job to the foreground.
+    given as an argument, bring that job to the foreground. Additionally,
+    specify "+" for the most recent job and "-" for the second most recent job.
     """
 
     _clear_dead_jobs()
@@ -251,9 +354,15 @@ def fg(args, stdin=None):
         act = tasks[0]  # take the last manipulated task by default
     elif len(args) == 1:
         try:
-            act = int(args[0])
-        except ValueError:
+            if args[0] == '+':  # take the last manipulated task
+                act = tasks[0]
+            elif args[0] == '-':  # take the second to last manipulated task
+                act = tasks[1]
+            else:
+                act = int(args[0])
+        except (ValueError, IndexError):
             return '', 'Invalid job: {}\n'.format(args[0])
+
         if act not in builtins.__xonsh_all_jobs__:
             return '', 'Invalid job: {}\n'.format(args[0])
     else:
